/*
	Ejemplo de visualizacin en Modo 4 (260x160 a 256 colores)

	Avelino Herrera Morales, 2005

	Basado en los ejemplos propuestos en http://belogic.com/gba
	He utilizado la toolchain descrita en http://linux.gbadev.org basada en binutils/gcc/newlib
	Cuidado con el fichero crt0.o, ya que debe tener habilitadas las interrupciones (http://devrs.com/gba).
*/

#include <string.h>
#include "gba.h"
#include "8x8font.h"
#include "mary_yo.h"

#define  BG0_ENABLE      0x100    //these are just the flags for enabling backgrounds and sprites
#define  BG1_ENABLE      0x200
#define  BG2_ENABLE      0x400
#define  BG3_ENABLE      0x800
#define  OBJ_ENABLE      0x1000
#define  WIN0_ENABLE     0x2000
#define  WIN1_ENABLE     0x4000
#define  WINOBJ_ENABLE   0x8000


#define  OBJ_MAP_LINEAR  0x40     //I'll talk about this when i talk about hardware sprites
#define  OBJ_MAP_2D      0x0
#define  BACK_BUFFER     0x10     //this is the flag that controlls wich buffer is being rendered


#define  MODE0           0x0  /* modos grficos de la GBA */
#define  MODE1           0x1
#define  MODE2           0x2
#define  MODE3           0x3
#define  MODE4           0x4
#define  MODE5           0x5

#define  RGB16(r,g,b)  ((r) | ((g) << 5) | ((b) << 10))


u16* video_buffer_0 = (u16 *) 0x6000000;
u16* video_buffer_1 = (u16 *) 0x600A000;
u16* bg_palette_mem = (u16 *) 0x5000000;


void put_pixel(int x, int y, u8 color) {
	int offset = (y * 120) + (x >> 1);
	u16 value = video_buffer_0[offset];
	if ((x % 2) != 0)
		value = (value & 0x00FF) | (((u16) color) << 8);
	else
		value = (value & 0xFF00) | (((u16) color) & 0x00FF);
	video_buffer_0[offset] = value;
}

void puts(u16 x, u16 y, u8 color, char *s) {
	int i = 0;
	while (s[i] != 0) {
		int c = s[i];
		if ((c >= 0x21) || (c <= 126)) {
			int font_offset = (c - 0x21) << 3;
			int y_aux, x_aux;
			for (y_aux = 0; y_aux < 8; y_aux++) {
				int byte = font8x8[font_offset + y_aux];
				for (x_aux = 7; x_aux >= 0; x_aux--) {
					if ((byte >> x_aux) & 0x01)
						put_pixel(x + (7 - x_aux), y + y_aux, color);
				}
			}
		}
		x += 8;
		i++;
	}
}


int AgbMain(void) {
	int i;

	/* activamos el modo grfico 4 */
	REG_DISPCNT = MODE4 | BG2_ENABLE;
	/* cargamos la paleta */
	for (i = 0; i < 256; i++)
		bg_palette_mem[i] = imagePalette[i];

	/* y finalmente cargamos la imagen */
	memcpy(video_buffer_0, imageData, image_WIDTH * image_HEIGHT);

	/* ponemos algunas cadenas de texto */
	puts(10, 130, 10, "Mari Carmen");
	puts(140, 40, 10, "Avelino");

	/* bucle infinito */
	while (1)
		;
}

